@@ -1,29 +1,67 @@
-use core::{marker::PhantomData, mem};
+use core::{
+    fmt::{self, Debug},
+    marker::PhantomData,
+    mem,
+};
 
 use bytemuck::{Pod, PodCastError};
 
-use crate::{ImplsCopy, ImplsZeroable};
+use crate::{IsCopy, IsZeroable};
+
+/// Constructs an [`IsPod<$T>`](struct@crate::IsPod),
+/// requires `$T:`[`Pod`](trait@Pod).
+///
+/// This has an optional type argument (`$T`) that defaults to
+/// infering the type if not passed.
+///
+/// # Example
+///
+/// ```rust
+/// use constmuck::{IsPod, cast, infer};
+///
+/// // transmuting `i16` to `u16`
+/// const FOO: u16 = cast(-1i16, (IsPod!(), IsPod!()));
+/// const BAR: u16 = cast(-1, (IsPod!(i16), IsPod!(u16)));
+/// // A more concise way to call `constmuck::cast` when the types are inferred.
+/// const BAZ: u16 = cast(-1i16, infer!());
+///
+/// assert_eq!(FOO, u16::MAX);
+/// assert_eq!(BAR, u16::MAX);
+/// assert_eq!(BAZ, u16::MAX);
+///
+/// ```
+#[macro_export]
+macro_rules! IsPod {
+    () => {
+        <$crate::IsPod<_> as $crate::Infer>::INFER
+    };
+    ($T:ty) => {
+        <$crate::IsPod<$T> as $crate::Infer>::INFER
+    };
+}
 
 mod __ {
     use super::*;
 
-    /// Encodes a `T:`[`Pod`] bound as a value,
-    /// avoids requiring (unstable as of 2021) trait bounds in `const fn`s.
+    /// Encodes a `T:`[`Pod`](trait@Pod) bound as a value.
     ///
     /// # Example
     ///
     /// ```rust
-    /// use constmuck::{ImplsPod, cast, cast_ref_alt, cast_slice_alt, infer};
+    /// use constmuck::{IsPod, cast, cast_ref_alt, cast_slice_alt, infer};
     ///
     /// {
     ///     // transmuting `i16` to `u16`
-    ///     const FOO1: u16 = cast(-1i16, (ImplsPod::NEW, ImplsPod::NEW));
-    ///    
-    ///     // The same as the above constant
-    ///     const FOO2: u16 = cast(-1i16, infer!());
+    ///     // The four lines below are equivalent
+    ///     const FOO1: u16 = cast(-1i16, (IsPod::NEW, IsPod::NEW));
+    ///     const FOO2: u16 = cast(-1i16, (IsPod!(), IsPod!()));
+    ///     const FOO3: u16 = cast(-1, (IsPod!(i16), IsPod!(u16)));
+    ///     const FOO4: u16 = cast(-1i16, infer!());
     ///    
     ///     assert_eq!(FOO1, u16::MAX);
     ///     assert_eq!(FOO2, u16::MAX);
+    ///     assert_eq!(FOO3, u16::MAX);
+    ///     assert_eq!(FOO4, u16::MAX);
     /// }
     ///
     /// {
@@ -41,41 +79,52 @@ mod __ {
     /// }
     ///
     /// ```
-    pub struct ImplsPod<T> {
-        pub impls_copy: ImplsCopy<T>,
-        pub impls_zeroable: ImplsZeroable<T>,
-        _private: PhantomData<fn() -> T>,
+    pub struct IsPod<T> {
+        /// All types that are [`Pod`](trait@Pod) are [`Copy`]
+        pub is_copy: IsCopy<T>,
+        /// All types that are [`Pod`](trait@Pod) are [`Zeroable`](trait@bytemuck::Zeroable)
+        pub is_zeroable: IsZeroable<T>,
+        // The lifetime of `T` is invariant,
+        // just in case that it's unsound for lifetimes to be co/contravariant.
+        _private: PhantomData<fn(T) -> T>,
     }
 
-    impl<T> Copy for ImplsPod<T> {}
+    impl<T> Debug for IsPod<T> {
+        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            f.write_str("IsPod")
+        }
+    }
 
-    impl<T> Clone for ImplsPod<T> {
+    impl<T> Copy for IsPod<T> {}
+
+    impl<T> Clone for IsPod<T> {
         fn clone(&self) -> Self {
             *self
         }
     }
 
-    impl<T: Pod> ImplsPod<T> {
-        /// Constructs an `ImplsPod`
+    impl<T: Pod> IsPod<T> {
+        /// Constructs an `IsPod`.
         ///
-        /// You can also use the [`infer`] macro to construct `ImplsPod` arguments.
+        /// You can also use the [`IsPod`](macro@crate::IsPod) or [`ìnfer`](macro@crate::infer)
+        /// macros to construct `IsPod` arguments.
         pub const NEW: Self = Self {
-            impls_copy: ImplsCopy::NEW,
-            impls_zeroable: ImplsZeroable::NEW,
+            is_copy: IsCopy::NEW,
+            is_zeroable: IsZeroable::NEW,
             _private: PhantomData,
         };
     }
 
-    impl<T> ImplsPod<T> {
+    impl<T> IsPod<T> {
         const __NEW_UNCHECKED__: Self = unsafe {
             Self {
-                impls_copy: ImplsCopy::new_unchecked(),
-                impls_zeroable: ImplsZeroable::new_unchecked(),
+                is_copy: IsCopy::new_unchecked(),
+                is_zeroable: IsZeroable::new_unchecked(),
                 _private: PhantomData,
             }
         };
 
-        /// Constructs an `ImplsPod<T>` without checking that `T` implements [`Pod`].
+        /// Constructs an `IsPod<T>` without checking that `T` implements [`Pod`](trait@Pod).
         ///
         /// # Safety
         ///
@@ -83,13 +132,13 @@ mod __ {
         /// [safety requirements of `Pod`](bytemuck::Pod#safety)
         ///
         /// ```rust
-        /// use constmuck::{ImplsPod, cast, infer};
+        /// use constmuck::{IsPod, cast};
         ///
         /// #[repr(transparent)]
         /// struct Foo([u8; 4]);
         ///
         /// unsafe{
-        ///     let bounds = (ImplsPod::new_unchecked(), ImplsPod::new_unchecked());
+        ///     let bounds = (IsPod::new_unchecked(), IsPod::new_unchecked());
         ///     assert_eq!(cast::<u32, Foo>(12345678, bounds).0, 12345678u32.to_ne_bytes());
         /// }
         /// ```
@@ -99,15 +148,15 @@ mod __ {
         }
     }
 }
-pub use __::ImplsPod;
+pub use __::IsPod;
 
-impl<T: Pod> crate::Infer for ImplsPod<T> {
+impl<T: Pod> crate::Infer for IsPod<T> {
     const INFER: Self = Self::NEW;
 }
 
-/// For casting `T` into `U`
+/// Casts `T` into `U`
 ///
-/// Requires both `T` and `U` to implement [`Pod`].
+/// Requires both `T` and `U` to implement [`Pod`](trait@Pod).
 ///
 /// # Panics
 ///
@@ -123,20 +172,22 @@ impl<T: Pod> crate::Infer for ImplsPod<T> {
 /// assert_eq!(LE_BYTES, 0xAB1E_BEEF_u32.to_le_bytes());
 ///
 /// ```
-pub const fn cast<T, U>(from: T, _bounds: (ImplsPod<T>, ImplsPod<U>)) -> U {
+pub const fn cast<T, U>(from: T, _bounds: (IsPod<T>, IsPod<U>)) -> U {
     unsafe {
         if mem::size_of::<T>() != mem::size_of::<U>() {
-            let x = mem::size_of::<T>();
-            let _: () = [/* the size of T and U is not the same */][x];
+            crate::__priv_utils::unequal_size_panic(mem::size_of::<T>(), mem::size_of::<U>())
         }
 
+        // safety: the `_bounds` parameter guarantees that both `T` and `U`
+        // contain no padding and are valid for all bitpatterns.
+        // They are both guaranteed to be the same size by the above conditional.
         __priv_transmute!(T, U, from)
     }
 }
 
-/// For casting `T` into `U`
+/// Tries to cast `T` into `U`
 ///
-/// Requires both `T` and `U` to implement [`Pod`].
+/// Requires both `T` and `U` to implement [`Pod`](trait@Pod).
 ///
 /// # Errors
 ///
@@ -158,10 +209,13 @@ pub const fn cast<T, U>(from: T, _bounds: (ImplsPod<T>, ImplsPod<U>)) -> U {
 /// ```
 pub const fn try_cast<T, U>(
     from: T,
-    _bounds: (ImplsPod<T>, ImplsPod<U>),
+    _bounds: (IsPod<T>, IsPod<U>),
 ) -> Result<U, crate::PodCastError> {
     unsafe {
         if mem::size_of::<T>() == mem::size_of::<U>() {
+            // safety: the `_bounds` parameter guarantees that both `T` and `U`
+            // contain no padding and are valid for all bitpatterns.
+            // They are both guaranteed the same size in this branch.
             Ok(__priv_transmute!(T, U, from))
         } else {
             // Pod requires types to be Copy, so this never causes a leak
@@ -173,7 +227,7 @@ pub const fn try_cast<T, U>(
 
 /// Cast a `&T` to `&U`
 ///
-/// Requires both `T` and `U` to implement [`Pod`].
+/// Requires both `T` and `U` to implement [`Pod`](trait@Pod).
 ///
 /// # Panics
 ///
@@ -184,8 +238,8 @@ pub const fn try_cast<T, U>(
 /// # Difference with `bytemuck`
 ///
 /// This function requires `T` to have an alignment larger or equal to `U`,
-/// while [`bytemuck::cast_ref`] only requires `from` to happen to be aligned
-/// to `U`.
+/// while [`bytemuck::cast_ref`] only requires the `from` reference
+/// to happen to be aligned to `U`.
 ///
 /// # Example
 ///
@@ -198,29 +252,32 @@ pub const fn try_cast<T, U>(
 /// assert_eq!(U8[1], 0);
 ///
 /// ```
-pub const fn cast_ref_alt<T, U>(from: &T, bounds: (ImplsPod<T>, ImplsPod<U>)) -> &U {
+pub const fn cast_ref_alt<T, U>(from: &T, bounds: (IsPod<T>, IsPod<U>)) -> &U {
     match try_cast_ref_alt(from, bounds) {
         Ok(x) => x,
         Err(PodCastError::TargetAlignmentGreaterAndInputNotAligned) => {
-            let x = mem::size_of::<T>();
-            [/* the alignment of T is larger than U */][x]
+            crate::__priv_utils::incompatible_alignment_panic(
+                mem::align_of::<T>(),
+                mem::align_of::<U>(),
+            )
         }
         Err(PodCastError::SizeMismatch | _) => {
-            let x = mem::size_of::<T>();
-            [/* the size of T and U is not the same */][x]
+            crate::__priv_utils::unequal_size_panic(mem::size_of::<T>(), mem::size_of::<U>())
         }
     }
 }
 
-/// Cast a `&T` to `&U`
+/// Tries to cast `&T` to `&U`
 ///
-/// Requires both `T` and `U` to implement [`Pod`].
+/// Requires both `T` and `U` to implement [`Pod`](trait@Pod).
 ///
 /// # Errors
 ///
 /// This function returns errors in these cases:
 /// - The alignment of `T` is larger than `U`, returning a
 /// `Err(PodCastError::TargetAlignmentGreaterAndInputNotAligned)`.
+/// <br>(using this instead of `PodCastError::AlignmentMismatch` because that
+/// is not returned by [`bytemuck::try_cast_ref`])
 ///
 /// - The size of `T` is not equal to `U`, returning a
 /// `Err(PodCastError::SizeMismatch)`.
@@ -228,8 +285,8 @@ pub const fn cast_ref_alt<T, U>(from: &T, bounds: (ImplsPod<T>, ImplsPod<U>)) ->
 /// # Difference with `bytemuck`
 ///
 /// This function requires `T` to have an alignment larger or equal to `U`,
-/// while [`bytemuck::try_cast_ref`] only requires `from` to happen to be aligned
-/// to `U`.
+/// while [`bytemuck::try_cast_ref`] only requires the `from` reference
+/// to happen to be aligned to `U`.
 ///
 /// # Example
 ///
@@ -248,7 +305,7 @@ pub const fn cast_ref_alt<T, U>(from: &T, bounds: (ImplsPod<T>, ImplsPod<U>)) ->
 /// ```
 pub const fn try_cast_ref_alt<T, U>(
     from: &T,
-    _bounds: (ImplsPod<T>, ImplsPod<U>),
+    _bounds: (IsPod<T>, IsPod<U>),
 ) -> Result<&U, crate::PodCastError> {
     unsafe {
         if mem::align_of::<T>() < mem::align_of::<U>() {
@@ -256,6 +313,11 @@ pub const fn try_cast_ref_alt<T, U>(
         } else if mem::size_of::<T>() != mem::size_of::<U>() {
             Err(PodCastError::SizeMismatch)
         } else {
+            // safety: the `_bounds` parameter guarantees that both `T` and `U`
+            // contain no padding and are valid for all bitpatterns.
+            //
+            // They are both guaranteed the same size in this branch,
+            // and T is at least as aligned as U.
             Ok(__priv_transmute_ref!(T, U, from))
         }
     }
