@@ -1,13 +1,15 @@
-//! Functions for wrapping/peeling types that implement [`TransparentWrapper`]
+//! Functions for wrapping/peeling types that implement
+//! [`TransparentWrapper`](trait@TransparentWrapper).
 //!
-//! Related: [`ImplsTransparentWrapper`] type and [`infer_tw`] macro.
+//! Related: [`IsTransparentWrapper`](crate::IsTransparentWrapper) type and
+//! [IsTW](crate::IsTW) macro.
 //!
 //! # Example
 //!
 //! Tranmuting between arrays of values and arrays of wrappers.
 //!
 //! ```rust
-//! use constmuck::{infer_tw, wrapper};
+//! use constmuck::{IsTW, wrapper};
 //!
 //! #[derive(Debug, PartialEq)]
 //! #[repr(transparent)]
@@ -17,75 +19,79 @@
 //!
 //! // `[u8; 3]` to `[Foo<u8>; 3]`
 //! {
-//!     const ARR: [Foo<u8>; 3] = wrapper::wrap([3, 5, 8], infer_tw!().array());
+//1     // Transmute `[u8; 3]` to `[Foo<u8>; 3]`
+//1     //
+//1     // The `.array()` is currently required to cast arrays of values into arrays of
+//1     // wrappers around those values.
+//!     const ARR: [Foo<u8>; 3] = wrapper::wrap([3, 5, 8], IsTW!(Foo<u8>, u8).array());
 //!
 //!     assert_eq!(ARR, [Foo(3), Foo(5), Foo(8)]);
 //!     // How to use `wrap` without relying on return type inference:
-//!     // `infer_tw!(Foo<_>)` is required because any type can implement comparison with `Foo`.
 //!     assert_eq!(
-//!         wrapper::wrap([13, 21, 34], infer_tw!(Foo<_>).array()),
+//!         wrapper::wrap([13, 21, 34], IsTW!(Foo<u8>, u8).array()),
 //!         [Foo(13), Foo(21), Foo(34)],
 //!     );
 //! }
 //!
 //! // `[Foo<u8>; 3]` to `[u8; 3]`
 //! {
-//!     const ARR: [u8; 3] = wrapper::peel([Foo(3), Foo(5), Foo(8)], infer_tw!().array());
+//!     const ARR: [u8; 3] = wrapper::peel([Foo(3), Foo(5), Foo(8)], IsTW!(Foo<u8>, u8).array());
 //!
 //!     assert_eq!(ARR, [3, 5, 8]);
 //!     assert_eq!(
-//!         wrapper::peel([Foo(13), Foo(21), Foo(34)], infer_tw!(Foo<_>).array()),
+//!         wrapper::peel([Foo(13), Foo(21), Foo(34)], IsTW!(Foo<u16>, u16).array()),
 //!         [13, 21, 34],
 //!     );
 //! }
 //!
 //! // `&[u8; 3]` to `&[Foo<u8>; 3]`
 //! {
-//!     const REF_ARR: &[Foo<u8>; 3] = wrapper::wrap_ref(&[3, 5, 8], infer_tw!().array());
+//!     const REF_ARR: &[Foo<u8>; 3] = wrapper::wrap_ref(&[3, 5, 8], IsTW!(Foo<u8>, u8).array());
 //!
 //!     assert_eq!(REF_ARR, &[Foo(3), Foo(5), Foo(8)]);
 //!     assert_eq!(
-//!         wrapper::wrap_ref(&[13, 21, 34], infer_tw!(Foo<_>).array()),
+//!         wrapper::wrap_ref(&[13, 21, 34], IsTW!(Foo<i32>, i32).array()),
 //!         &[Foo(13), Foo(21), Foo(34)],
 //!     );    
 //! }
 //!
 //! // `&[Foo<u8>; 3]` to `&[u8; 3]`
 //! {
-//!     const REF_ARR: &[u8; 3] = wrapper::peel_ref(&[Foo(3), Foo(5), Foo(8)], infer_tw!().array());
+//!     const REF_ARR: &[u8; 3] = wrapper::peel_ref(&[Foo(3), Foo(5), Foo(8)], IsTW!().array());
 //!
 //!     assert_eq!(REF_ARR, &[3, 5, 8]);
 //!     assert_eq!(
-//!         wrapper::peel_ref(&[Foo(13), Foo(21), Foo(34)], infer_tw!(Foo<_>).array()),
+//!         wrapper::peel_ref(&[Foo(13), Foo(21), Foo(34)], IsTW!(Foo<_>).array()),
 //!         &[13, 21, 34],
 //!     );    
 //! }
 //!
 //! ```
 //!
-
 use bytemuck::TransparentWrapper;
 
-use crate::{Infer, TransmutableInto};
+use core::fmt::{self, Debug};
+
+use crate::Infer;
 
 #[doc(no_inline)]
-pub use crate::{infer_tw, ImplsTransparentWrapper};
+pub use crate::IsTransparentWrapper;
 
-/// Constructs an [`ImplsTransparentWrapper`],
+#[doc(no_inline)]
+pub use crate::IsTW;
+
+/// Constructs an [`IsTransparentWrapper<$Outer, $Inner>`](crate::IsTransparentWrapper),
+/// requiring that `$Outer` implements [`TransparentWrapper`]`<$Inner>`.
 ///
-/// Most useful over [`infer`] to:
-/// - Call methods on the [`ImplsTransparentWrapper`].
-/// - Access the [`from_inner`](ImplsTransparentWrapper::from_inner) field
-/// - Access the [`into_inner`](ImplsTransparentWrapper::into_inner) field.
-///
-/// Related: [`wrapper`](crate::wrapper) module
+/// This has two optional type arguments (`$Outer` and `$Inner`) that default to
+/// infering the type if not passed.
 ///
 /// # Example
 ///
 /// ### `constmuck::wrapper` functions
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -95,104 +101,74 @@ pub use crate::{infer_tw, ImplsTransparentWrapper};
 ///
 /// {
 ///     // Transmute `ì8` to `Foo<i8>`
-///     const WRAP_VAL_ONE: Foo<i8> = wrapper::wrap(3, infer_tw!());
+///     const WRAP_VAL_ONE: Foo<i8> = wrapper::wrap(3, IsTW!());
+///     
+///     assert_eq!(WRAP_VAL_ONE, Foo(3));
+///     assert_eq!(wrapper::wrap(3, IsTW!(Foo<i8>)), Foo(3));
+///     assert_eq!(wrapper::wrap(3, IsTW!(Foo<i8>, i8)), Foo(3));
+///     
 ///     
 ///     // Transmute `[u8; 3]` to `[Foo<u8>; 3]`
 ///     //
-///     // The `.array()` is required to cast arrays of values into arrays of
+///     // The `.array()` is currently required to cast arrays of values into arrays of
 ///     // wrappers around those values.
-///     const WRAP_VAL_ARR: [Foo<u8>; 3] = wrapper::wrap([5, 8, 13], infer_tw!().array());
-///     
-///     assert_eq!(WRAP_VAL_ONE, Foo(3));
-///     assert_eq!(wrapper::wrap(3, infer_tw!(Foo<i8>)), Foo(3));
-///     assert_eq!(wrapper::wrap(3, infer_tw!(Foo<i8>, i8)), Foo(3));
+///     const WRAP_VAL_ARR: [Foo<u8>; 3] = wrapper::wrap([5, 8, 13], IsTW!(Foo<u8>, u8).array());
 ///     
 ///     assert_eq!(WRAP_VAL_ARR, [Foo(5), Foo(8), Foo(13)]);
 ///     assert_eq!(
-///         wrapper::wrap([5, 8, 13], infer_tw!(Foo<u8>).array()),
+///         wrapper::wrap([5, 8, 13], IsTW!(Foo<u8>, u8).array()),
 ///         [Foo(5), Foo(8), Foo(13)],
 ///     );
 ///     assert_eq!(
-///         wrapper::wrap([5, 8, 13], infer_tw!(Foo<u8>, u8).array()),
+///         wrapper::wrap([5, 8, 13], IsTW!(Foo<u8>, u8).array()),
 ///         [Foo(5), Foo(8), Foo(13)],
 ///     );
 /// }
 /// {
 ///     // Transmute `&i8` to `&Foo<i8>`
-///     const WRAP_REF_ONE: &Foo<i8> = wrapper::wrap_ref(&3, infer_tw!());
+///     const WRAP_REF_ONE: &Foo<i8> = wrapper::wrap_ref(&3, IsTW!());
+///     assert_eq!(WRAP_REF_ONE, &Foo(3));
 ///     
 ///     // Transmute `&[u8; 3]` to `&[Foo<u8>; 3]`
-///     const WRAP_REF_ARR: &[Foo<u8>; 3] = wrapper::wrap_ref(&[5, 8, 13], infer_tw!().array());
-///     
-///     assert_eq!(WRAP_REF_ONE, &Foo(3));
+///     const WRAP_REF_ARR: &[Foo<u8>; 3] = wrapper::wrap_ref(&[5, 8, 13], IsTW!().array());
 ///     assert_eq!(WRAP_REF_ARR, &[Foo(5), Foo(8), Foo(13)]);
 /// }
 /// {
 ///     // Transmute `&[i8]` to `&[Foo<i8>]`
-///     const WRAP_SLICE: &[Foo<i8>] = wrapper::wrap_slice(&[21, 34, 55], infer_tw!());
+///     const WRAP_SLICE: &[Foo<i8>] = wrapper::wrap_slice(&[21, 34, 55], IsTW!());
 ///     
 ///     assert_eq!(WRAP_SLICE, &[Foo(21), Foo(34), Foo(55)]);
 /// }
 /// ```
 ///
-/// ### `constmuck::transmutable` functions
-///
-/// ```
-/// use constmuck::{
-///     transmutable::transmute_ref,
-///     infer_tw,
-/// };
-///
-/// use std::num::Wrapping;
-///
-/// {
-///     // Casting `&Wrapping<u8>` to `&u8`,
-///     //
-///     // `infer_tw!()` constructs an `ImplsTransparentWrapper`,
-///     // whose `into_inner` field allows casting from a wrapper into the value in it.
-///     const UNWRAPPED: &u8 = transmute_ref(&Wrapping(5), infer_tw!().into_inner);
-///     assert_eq!(*UNWRAPPED, 5);
-/// }
-///
-/// {
-///     // Casting `&u8` to `&Wrapping<u8>`
-///     //
-///     // `infer_tw!()` constructs an `ImplsTransparentWrapper`,
-///     // whose `from_inner` field allows casting from a value into a wrapper around it.
-///     const WRAPPED: &Wrapping<u8> = transmute_ref(&7, infer_tw!().from_inner);
-///    
-///     assert_eq!(*WRAPPED, Wrapping(7));
-/// }
-///
-/// ```
+/// [`TransparentWrapper`]: trait@TransparentWrapper
 #[macro_export]
-macro_rules! infer_tw {
+macro_rules! IsTW {
     () => {
-        <$crate::ImplsTransparentWrapper<_, _> as $crate::Infer>::INFER
+        <$crate::IsTransparentWrapper<_, _> as $crate::Infer>::INFER
     };
-    ($outer:ty $(,)*) => {
-        <$crate::ImplsTransparentWrapper<$outer, _> as $crate::Infer>::INFER
+    ($Outer:ty $(,)*) => {
+        <$crate::IsTransparentWrapper<$Outer, _> as $crate::Infer>::INFER
     };
-    ($outer:ty, $inner:ty $(,)*) => {
-        <$crate::ImplsTransparentWrapper<$outer, $inner> as $crate::Infer>::INFER
+    ($Outer:ty, $Inner:ty $(,)*) => {
+        <$crate::IsTransparentWrapper<$Outer, $Inner> as $crate::Infer>::INFER
     };
 }
 
-pub(crate) mod impls_tw {
+pub(crate) mod is_tw {
     use super::*;
 
-    /// Encodes a `T:`[`TransparentWrapper`] bound as a value,
-    /// avoids requiring (unstable as of 2021) trait bounds in `const fn`s.
+    /// Encodes a `Outer:`[`TransparentWrapper`]`<Inner>` bound as a value.
     ///
-    /// Constructible with [`NEW`](Self::NEW) associated constant,
-    /// or [`infer_tw`] macro.
+    /// Related: [`IsTW`](macro@crate::IsTW) macro, [`wrapper`](crate::wrapper) module.
     ///
-    /// Related: [`wrapper`](crate::wrapper) module.
+    /// You can also [construct an `IsTransparentWrapper<T, T>`](Self::IDENTITY),
+    /// which allows using a type wherever transparent wrappers to it are expected.
     ///
     /// # Example
     ///
     /// ```rust
-    /// use constmuck::{infer_tw, wrapper};
+    /// use constmuck::{IsTW, wrapper};
     ///
     /// #[derive(Debug, PartialEq)]
     /// #[repr(transparent)]
@@ -203,80 +179,114 @@ pub(crate) mod impls_tw {
     /// {
     ///     // Casting from `&&str` to `&This<&str>`
     ///     //
-    ///     // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-    ///     const WRAPPED: &This<&str> = wrapper::wrap_ref(&"hi", infer_tw!());
+    ///     // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+    ///     const WRAPPED: &This<&str> = wrapper::wrap_ref(&"hi", IsTW!());
     ///     assert_eq!(*WRAPPED, This("hi"));
     /// }
     ///
     /// {
     ///     // Casting from `&This<&str>` to `&&str`
-    ///     const UNWRAPPED: &&str = wrapper::peel_ref(&This("hello"), infer_tw!());
+    ///     const UNWRAPPED: &&str = wrapper::peel_ref(&This("hello"), IsTW!());
     ///     assert_eq!(*UNWRAPPED, "hello");
     /// }
     ///
     /// {
     ///     // Casting from `[u64; 2]` to `[This<u64>; 2]`
     ///     //
-    ///     // The `.array()` is required to cast arrays of values into arrays of
+    ///     // The `.array()` is currently required to cast arrays of values into arrays of
     ///     // wrappers around those values.
-    ///     const WRAPPED_ARR: [This<u64>; 2] = wrapper::wrap([9, 99], infer_tw!().array());
+    ///     const WRAPPED_ARR: [This<u64>; 2] =
+    ///         wrapper::wrap([9, 99], IsTW!(This<u64>, u64).array());
+    ///
     ///     assert_eq!(WRAPPED_ARR, [This(9), This(99)]);
     /// }
     ///
     /// {
     ///     // Casting from `[This<i8>; 2]` to `[i8; 2]`
     ///     //
-    ///     // `.array()` also allows casting arrays of wrappers into
+    ///     // `.array()` allows casting arrays of wrappers into
     ///     // arrays of the values inside those wrappers, using the `peel*` functions.
     ///     const UNWRAPPED_ARR: [i8; 2] =
-    ///         wrapper::peel([This(2), This(22)], infer_tw!().array());
+    ///         wrapper::peel([This(2), This(22)], IsTW!(This<i8>, i8).array());
+    ///
     ///     assert_eq!(UNWRAPPED_ARR, [2, 22]);
     /// }
     ///
     /// ```
-    pub struct ImplsTransparentWrapper<Outer: ?Sized, Inner: ?Sized> {
-        pub from_inner: TransmutableInto<Inner, Outer>,
-        pub into_inner: TransmutableInto<Outer, Inner>,
+    ///
+    /// [`TransparentWrapper`]: trait@TransparentWrapper
+    #[non_exhaustive]
+    pub struct IsTransparentWrapper<Outer: ?Sized, Inner: ?Sized> {
         #[doc(hidden)]
         pub _transparent_wrapper_proof: constmuck_internal::TransparentWrapperProof<Outer, Inner>,
     }
 
-    impl<Outer: ?Sized, Inner: ?Sized> Copy for ImplsTransparentWrapper<Outer, Inner> {}
+    impl<Outer: ?Sized, Inner: ?Sized> Debug for IsTransparentWrapper<Outer, Inner> {
+        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+            f.write_str("IsTransparentWrapper")
+        }
+    }
 
-    impl<Outer: ?Sized, Inner: ?Sized> Clone for ImplsTransparentWrapper<Outer, Inner> {
+    impl<Outer: ?Sized, Inner: ?Sized> Copy for IsTransparentWrapper<Outer, Inner> {}
+
+    impl<Outer: ?Sized, Inner: ?Sized> Clone for IsTransparentWrapper<Outer, Inner> {
         fn clone(&self) -> Self {
             *self
         }
     }
 
-    impl<Outer: ?Sized, Inner: ?Sized> ImplsTransparentWrapper<Outer, Inner>
+    impl<Outer: ?Sized, Inner: ?Sized> IsTransparentWrapper<Outer, Inner>
     where
         Outer: TransparentWrapper<Inner>,
     {
-        /// Constructs an `ImplsTransparentWrapper`
+        /// Constructs an `IsTransparentWrapper`
         ///
-        /// You can also use the [`infer_tw`] macro to construct `ImplsTransparentWrapper` arguments.
+        /// You can also use the [`IsTW`](macro@crate::IsTW) macro to
+        /// construct `IsTransparentWrapper` arguments.
         pub const NEW: Self = unsafe {
             Self {
-                from_inner: TransmutableInto::new_unchecked(),
-                into_inner: TransmutableInto::new_unchecked(),
                 _transparent_wrapper_proof:
                     constmuck_internal::TransparentWrapperProof::new_unchecked(),
             }
         };
     }
 
-    impl<Outer: ?Sized, Inner: ?Sized> ImplsTransparentWrapper<Outer, Inner> {
+    impl<T: ?Sized> IsTransparentWrapper<T, T> {
+        /// Constructs an `IsTransparentWrapper<T, T>`.
+        ///
+        /// # Example
+        ///
+        /// ```rust
+        /// use constmuck::{IsTransparentWrapper as IsTW, wrapper};
+        ///
+        /// use std::num::Wrapping;
+        ///
+        /// const fn add_u32<W>(left: u32, right: &W, is_u32: IsTW<W, u32>) -> u32 {
+        ///     left + *wrapper::peel_ref(right, is_u32)
+        /// }
+        ///
+        /// assert_eq!(add_u32(3, &Wrapping(5), IsTW::NEW), 8);
+        ///
+        /// assert_eq!(add_u32(8, &13, IsTW::IDENTITY), 21);
+        ///
+        /// ```
+        pub const IDENTITY: Self = unsafe {
+            Self {
+                _transparent_wrapper_proof:
+                    constmuck_internal::TransparentWrapperProof::new_unchecked(),
+            }
+        };
+    }
+
+    impl<Outer: ?Sized, Inner: ?Sized> IsTransparentWrapper<Outer, Inner> {
         const __NEW_UNCHECKED__: Self = unsafe {
             Self {
-                from_inner: TransmutableInto::new_unchecked(),
-                into_inner: TransmutableInto::new_unchecked(),
                 _transparent_wrapper_proof:
                     constmuck_internal::TransparentWrapperProof::new_unchecked(),
             }
         };
 
-        /// Constructs an `ImplsTransparentWrapper` without checking that
+        /// Constructs an `IsTransparentWrapper` without checking that
         /// `Outer` implements
         /// [`TransparentWrapper<Inner>`](bytemuck::TransparentWrapper).
         ///
@@ -290,65 +300,59 @@ pub(crate) mod impls_tw {
         pub const unsafe fn new_unchecked() -> Self {
             Self::__NEW_UNCHECKED__
         }
+    }
 
-        /// Constructs an `ImplsTransparentWrapper` from a pair
-        /// of [`TransmutableInto`] that allow transmuting between `Outer` and `Inner`
-        /// in both directions.
+    impl<Outer: ?Sized, Inner: ?Sized> IsTransparentWrapper<Outer, Inner> {
+        /// Combines an `IsTransparentWrapper` with another to allow
+        /// casting between `Outer` and `Nested`.
+        ///
+        /// Without this you'd have to do `Outer -> Inner -> Nested` casts.
         ///
         /// # Example
         ///
         /// ```rust
-        /// use constmuck::{ImplsTransparentWrapper, TransmutableInto, infer, wrapper};
+        /// use constmuck::{IsTW, IsTransparentWrapper, wrapper};
         ///
-        /// const ITW: ImplsTransparentWrapper<u8, i8> =
-        ///     ImplsTransparentWrapper::from_ti(
-        ///         TransmutableInto::pod(infer!()),
-        ///         TransmutableInto::pod(infer!()),
-        ///     );
+        /// use std::num::Wrapping;
         ///
-        /// const U8_VAL: u8 = wrapper::wrap(-1, ITW);
-        /// assert_eq!(U8_VAL, 255);
+        /// const FOO: IsTransparentWrapper<Bar<u32>, u32> = IsTW!().join(IsTW!());
         ///
-        /// const U8_REF: &u8 = wrapper::wrap_ref(&-2, ITW);
-        /// assert_eq!(U8_REF, &254);
+        /// // Equivalent to FOO, but passing the types explicitly to `IsTW`.
+        /// // Only the last joined `IsTW!(` needs to be passed the `$Inner` argument.
+        /// let foo = IsTW!(Bar<_>).join(IsTW!(Wrapping<u32>, u32));
         ///
-        /// const U8_SLICE: &[u8] = wrapper::wrap_slice(&[-3, 3], ITW);
-        /// assert_eq!(U8_SLICE, &[253, 3]);
+        /// assert_eq!(wrapper::wrap_ref(&5, FOO), &Bar(Wrapping(5)));
         ///
-        /// const I8_VAL: i8 = wrapper::peel(128, ITW);
-        /// assert_eq!(I8_VAL, -128);
+        /// assert_eq!(wrapper::wrap_ref(&8, foo), &Bar(Wrapping(8)));
         ///
-        /// const I8_REF: &i8 = wrapper::peel_ref(&129, ITW);
-        /// assert_eq!(I8_REF, &-127);
-        ///
-        /// const I8_SLICE: &[i8] = wrapper::peel_slice(&[254, 1], ITW);
-        /// assert_eq!(I8_SLICE, &[-2, 1]);
+        /// assert_eq!(
+        ///     wrapper::wrap_slice(&[13, 21], FOO),
+        ///     &[Bar(Wrapping(13)), Bar(Wrapping(21))]
+        /// );
         ///
         ///
+        /// #[derive(Debug, PartialEq)]
+        /// struct Bar<T>(Wrapping<T>);
+        ///
+        /// unsafe impl<T> constmuck::TransparentWrapper<Wrapping<T>> for Bar<T> {}
         ///
         /// ```
-        pub const fn from_ti(
-            from_inner: TransmutableInto<Inner, Outer>,
-            into_inner: TransmutableInto<Outer, Inner>,
-        ) -> Self {
-            Self {
-                from_inner,
-                into_inner,
-                _transparent_wrapper_proof: unsafe {
-                    constmuck_internal::TransparentWrapperProof::new_unchecked()
-                },
-            }
+        pub const fn join<Nested: ?Sized>(
+            self,
+            _other: IsTransparentWrapper<Inner, Nested>,
+        ) -> IsTransparentWrapper<Outer, Nested> {
+            IsTransparentWrapper::__NEW_UNCHECKED__
         }
     }
 
-    impl<Outer, Inner> ImplsTransparentWrapper<Outer, Inner> {
-        /// Turns a `ImplsTransparentWrapper<Outer, Inner>` into a
-        /// `ImplsTransparentWrapper<[Outer; LEN], [Inner; LEN]>`.
+    impl<Outer, Inner> IsTransparentWrapper<Outer, Inner> {
+        /// Turns a `IsTransparentWrapper<Outer, Inner>` into a
+        /// `IsTransparentWrapper<[Outer; LEN], [Inner; LEN]>`.
         ///
         /// # Example
         ///
         /// ```rust
-        /// use constmuck::{infer_tw, wrapper};
+        /// use constmuck::{IsTW, wrapper};
         ///
         /// #[derive(Debug, PartialEq)]
         /// #[repr(transparent)]
@@ -360,7 +364,7 @@ pub(crate) mod impls_tw {
         ///     // Casting from `[u32; 5]` to `[Xyz<u32>; 5]`
         ///     const ARR: [Xyz<u32>; 5] = wrapper::wrap(
         ///         [3, 5, 13, 34, 89],
-        ///         infer_tw!().array(),
+        ///         IsTW!(Xyz<u32>, u32).array(),
         ///     );
         ///    
         ///     assert_eq!(ARR, [Xyz(3), Xyz(5), Xyz(13), Xyz(34), Xyz(89)]);
@@ -370,7 +374,7 @@ pub(crate) mod impls_tw {
         ///     // Casting from `[Xyz<u32>; 5]` to `[u32; 5]`
         ///     const ARR: [u32; 5] = wrapper::peel(
         ///         [Xyz(3), Xyz(5), Xyz(13), Xyz(34), Xyz(89)],
-        ///         infer_tw!().array::<5>(),
+        ///         IsTW!(Xyz<u32>, u32).array::<5>(),
         ///     );
         ///    
         ///     assert_eq!(ARR, [3, 5, 13, 34, 89]);
@@ -379,10 +383,8 @@ pub(crate) mod impls_tw {
         #[inline(always)]
         pub const fn array<const LEN: usize>(
             self,
-        ) -> ImplsTransparentWrapper<[Outer; LEN], [Inner; LEN]> {
-            ImplsTransparentWrapper {
-                from_inner: self.from_inner.array(),
-                into_inner: self.into_inner.array(),
+        ) -> IsTransparentWrapper<[Outer; LEN], [Inner; LEN]> {
+            IsTransparentWrapper {
                 _transparent_wrapper_proof: unsafe {
                     constmuck_internal::TransparentWrapperProof::new_unchecked()
                 },
@@ -391,7 +393,7 @@ pub(crate) mod impls_tw {
     }
 }
 
-impl<Outer: ?Sized, Inner: ?Sized> Infer for ImplsTransparentWrapper<Outer, Inner>
+impl<Outer: ?Sized, Inner: ?Sized> Infer for IsTransparentWrapper<Outer, Inner>
 where
     Outer: TransparentWrapper<Inner>,
 {
@@ -408,7 +410,7 @@ where
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -419,32 +421,33 @@ where
 ///
 /// // Casting `&u32` to `Qux<u32>`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const VALUE: Qux<u32> = wrapper::wrap(3, infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const VALUE: Qux<u32> = wrapper::wrap(3, IsTW!());
 ///
 /// assert_eq!(VALUE, Qux(3));
 ///
-/// // `infer_tw!(Qux<_>)` is required because any type can implement comparison with `Qux`.
-/// assert_eq!(wrapper::wrap(3, infer_tw!(Qux<_>)), Qux(3));
+/// // `IsTW!(Qux<_>)` is required because any type can implement comparison with `Qux`.
+/// assert_eq!(wrapper::wrap(3, IsTW!(Qux<_>)), Qux(3));
 ///
 ///
 /// // Casting `[u32; 3]` to `[Qux<u32>; 3]`
 /// //
-/// // The `.array()` is required to cast arrays of values into arrays of
+/// // The `.array()` is currently required to cast arrays of values into arrays of
 /// // wrappers around those values.
-/// const ARR: [Qux<u32>; 3] = wrapper::wrap([5, 8, 13], infer_tw!().array());
+/// const ARR: [Qux<u32>; 3] = wrapper::wrap([5, 8, 13], IsTW!(Qux<u32>, u32).array());
 ///
 /// assert_eq!(ARR, [Qux(5), Qux(8), Qux(13)]);
 ///
 /// assert_eq!(
-///     wrapper::wrap([5, 8, 13], infer_tw!(Qux<_>).array()),
+///     wrapper::wrap([5, 8, 13], IsTW!(Qux<_>).array()),
 ///     [Qux(5), Qux(8), Qux(13)],
 /// );
 ///
 /// ```
-pub const fn wrap<Inner, Outer>(val: Inner, _: ImplsTransparentWrapper<Outer, Inner>) -> Outer {
+pub const fn wrap<Outer, Inner>(val: Inner, _bound: IsTransparentWrapper<Outer, Inner>) -> Outer {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe { __priv_transmute!(Inner, Outer, val) }
 }
 
@@ -459,7 +462,7 @@ pub const fn wrap<Inner, Outer>(val: Inner, _: ImplsTransparentWrapper<Outer, In
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -469,21 +472,22 @@ pub const fn wrap<Inner, Outer>(val: Inner, _: ImplsTransparentWrapper<Outer, In
 ///
 /// // Casting `&u32` to `&Foo<u32>`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const X: &Foo<u32> = wrapper::wrap_ref(&100, infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const X: &Foo<u32> = wrapper::wrap_ref(&100, IsTW!());
 ///
 /// assert_eq!(X, &Foo(100));
 ///
-/// // `infer_tw!(Foo<_>)` is required because any type can implement comparison with `Foo`.
-/// assert_eq!(wrapper::wrap_ref(&100, infer_tw!(Foo<_>)), &Foo(100));
+/// // `IsTW!(Foo<_>)` is required because any type can implement comparison with `Foo`.
+/// assert_eq!(wrapper::wrap_ref(&100, IsTW!(Foo<_>)), &Foo(100));
 ///
 /// ```
-pub const fn wrap_ref<Inner, Outer>(
+pub const fn wrap_ref<Outer, Inner>(
     reff: &Inner,
-    _: ImplsTransparentWrapper<Outer, Inner>,
+    _bound: IsTransparentWrapper<Outer, Inner>,
 ) -> &Outer {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe {
         __priv_transmute_ref! {Inner, Outer, reff}
     }
@@ -496,7 +500,7 @@ pub const fn wrap_ref<Inner, Outer>(
 /// ```rust
 /// pub const fn wrap_ref<Inner: ?Sized, Outer: ?Sized>(
 ///     reff: &Inner,
-///     impls_tw: constmuck::ImplsTransparentWrapper<Outer, Inner>
+///     is_tw: constmuck::IsTransparentWrapper<Outer, Inner>
 /// ) -> &Outer
 /// # { loop{} }
 /// ```
@@ -505,14 +509,15 @@ pub const fn wrap_ref<Inner, Outer>(
 /// [`TransparentWrapper<Inner>`](bytemuck::TransparentWrapper)
 ///
 /// Note that, because of how this macro is implemented,
-/// [`infer`] cannot be passed as the `impls_tw` argument.
-/// You must pass a type that's known to be an [`ImplsTransparentWrapper`] beforehand,
-/// eg: [`infer_tw!()`](crate::infer_tw), [`ImplsTransparentWrapper::NEW`].
+/// [`infer`] cannot be passed as the `is_tw` argument.
+/// You must pass a type that's known to be an
+/// [`IsTransparentWrapper`](crate::IsTransparentWrapper) beforehand,
+/// eg: [`IsTW!()`](macro@crate::IsTW), [`IsTransparentWrapper::NEW`].
 ///
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -522,13 +527,13 @@ pub const fn wrap_ref<Inner, Outer>(
 ///
 /// // Casting `&str` to `&Foo<str>`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const X: &Foo<str> = wrapper::wrap_ref!("world", infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const X: &Foo<str> = wrapper::wrap_ref!("world", IsTW!());
 ///
 /// assert_eq!(X.0, *"world");
 ///
-/// // `infer_tw!(Foo<_>)` is required because any type can implement comparison with `Foo`.
-/// assert_eq!(wrapper::wrap_ref!("huh", infer_tw!(Foo<_>)).0, *"huh");
+/// // `IsTW!(Foo<_>)` is required because any type can implement comparison with `Foo`.
+/// assert_eq!(wrapper::wrap_ref!("huh", IsTW!(Foo<_>)).0, *"huh");
 ///
 /// ```
 pub use constmuck_internal::wrapper_wrap_ref as wrap_ref;
@@ -541,7 +546,7 @@ pub use constmuck_internal::wrapper_wrap_ref as wrap_ref;
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -551,24 +556,25 @@ pub use constmuck_internal::wrapper_wrap_ref as wrap_ref;
 ///
 /// // Casting `&[&str]` to `&[Bar<&str>]`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const X: &[Bar<&str>] = wrapper::wrap_slice(&["hello", "world"], infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const X: &[Bar<&str>] = wrapper::wrap_slice(&["hello", "world"], IsTW!());
 ///
 /// assert_eq!(X, [Bar("hello"), Bar("world")]);
 ///
-/// // `infer_tw!(Bar<_>)` is required because any type can implement comparison with `Bar`.
+/// // `IsTW!(Bar<_>)` is required because any type can implement comparison with `Bar`.
 /// assert_eq!(
-///     wrapper::wrap_slice(&["hello", "world"], infer_tw!(Bar<_>)),
+///     wrapper::wrap_slice(&["hello", "world"], IsTW!(Bar<_>)),
 ///     [Bar("hello"), Bar("world")],
 /// );
 ///
 /// ```
-pub const fn wrap_slice<Inner, Outer>(
+pub const fn wrap_slice<Outer, Inner>(
     reff: &[Inner],
-    _: ImplsTransparentWrapper<Outer, Inner>,
+    _bound: IsTransparentWrapper<Outer, Inner>,
 ) -> &[Outer] {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe {
         __priv_transmute_slice! {Inner, Outer, reff}
     }
@@ -582,13 +588,13 @@ pub const fn wrap_slice<Inner, Outer>(
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// use std::num::Wrapping;
 ///
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const VALUE: u32 = wrapper::peel(Wrapping(3), infer_tw!());
-///
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const VALUE: u32 = wrapper::peel(Wrapping(3), IsTW!());
+/// assert_eq!(VALUE, 3);
 ///
 /// // Casting `[Wrapping<u32>; 3]` to `[u32; 3]`
 /// //
@@ -596,16 +602,15 @@ pub const fn wrap_slice<Inner, Outer>(
 /// // arrays of the values inside those wrappers.
 /// const ARR: [u32; 3] = wrapper::peel(
 ///     [Wrapping(5), Wrapping(8), Wrapping(13)],
-///     infer_tw!().array(),
+///     IsTW!(Wrapping<u32>, u32).array(),
 /// );
-///
-/// assert_eq!(VALUE, 3);
 /// assert_eq!(ARR, [5, 8, 13]);
 ///
 /// ```
-pub const fn peel<Inner, Outer>(val: Outer, _: ImplsTransparentWrapper<Outer, Inner>) -> Inner {
+pub const fn peel<Outer, Inner>(val: Outer, _bound: IsTransparentWrapper<Outer, Inner>) -> Inner {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe { __priv_transmute!(Outer, Inner, val) }
 }
 
@@ -620,7 +625,7 @@ pub const fn peel<Inner, Outer>(val: Outer, _: ImplsTransparentWrapper<Outer, In
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -630,18 +635,19 @@ pub const fn peel<Inner, Outer>(val: Outer, _: ImplsTransparentWrapper<Outer, In
 ///
 /// // Casting `&Foo<char>` to `&char`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const X: &char = wrapper::peel_ref(&Foo('@'), infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const X: &char = wrapper::peel_ref(&Foo('@'), IsTW!());
 ///
 /// assert_eq!(X, &'@');
 ///
 /// ```
-pub const fn peel_ref<Inner, Outer>(
+pub const fn peel_ref<Outer, Inner>(
     reff: &Outer,
-    _: ImplsTransparentWrapper<Outer, Inner>,
+    _bound: IsTransparentWrapper<Outer, Inner>,
 ) -> &Inner {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe {
         __priv_transmute_ref! {Outer, Inner, reff}
     }
@@ -654,7 +660,7 @@ pub const fn peel_ref<Inner, Outer>(
 /// ```rust
 /// pub const fn peel_ref<Inner: ?Sized, Outer: ?Sized>(
 ///     reff: &Outer,
-///     impls_tw: constmuck::ImplsTransparentWrapper<Outer, Inner>
+///     is_tw: constmuck::IsTransparentWrapper<Outer, Inner>
 /// ) -> &Inner
 /// # { loop{} }
 /// ```
@@ -663,14 +669,15 @@ pub const fn peel_ref<Inner, Outer>(
 /// [`TransparentWrapper<Inner>`](bytemuck::TransparentWrapper)
 ///
 /// Note that, because of how this macro is implemented,
-/// [`infer`] cannot be passed as the `impls_tw` argument.
-/// You must pass a type that's known to be an [`ImplsTransparentWrapper`] beforehand,
-/// eg: [`infer_tw!()`](crate::infer_tw), [`ImplsTransparentWrapper::NEW`].
+/// [`infer`] cannot be passed as the `is_tw` argument.
+/// You must pass a type that's known to be an
+/// [`IsTransparentWrapper`](crate::IsTransparentWrapper) beforehand,
+/// eg: [`IsTW!()`](macro@crate::IsTW), [`IsTransparentWrapper::NEW`].
 ///
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -683,8 +690,8 @@ pub const fn peel_ref<Inner, Outer>(
 ///
 ///     // Casting `&Foo<[u8]>` to `&[u8]`
 ///     //
-///     // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-///     wrapper::peel_ref!(x, infer_tw!())
+///     // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+///     wrapper::peel_ref!(x, IsTW!())
 /// };
 ///
 /// assert_eq!(X, [3, 5, 8, 13]);
@@ -700,7 +707,7 @@ pub use constmuck_internal::wrapper_peel_ref as peel_ref;
 /// # Example
 ///
 /// ```rust
-/// use constmuck::{infer_tw, wrapper};
+/// use constmuck::{IsTW, wrapper};
 ///
 /// #[derive(Debug, PartialEq)]
 /// #[repr(transparent)]
@@ -710,18 +717,19 @@ pub use constmuck_internal::wrapper_peel_ref as peel_ref;
 ///
 /// // Casting `&[Bar<&str>]` to `&[&str]`
 /// //
-/// // `infer_tw!()` is a more concise way to write `ImplsTransparentWrapper::NEW`
-/// const X: &[&str] = wrapper::peel_slice(&[Bar("hello"), Bar("world")], infer_tw!());
+/// // `IsTW!()` is a more concise way to write `IsTransparentWrapper::NEW`
+/// const X: &[&str] = wrapper::peel_slice(&[Bar("hello"), Bar("world")], IsTW!());
 ///
 /// assert_eq!(X, ["hello", "world"]);
 ///
 /// ```
-pub const fn peel_slice<Inner, Outer>(
+pub const fn peel_slice<Outer, Inner>(
     reff: &[Outer],
-    _: ImplsTransparentWrapper<Outer, Inner>,
+    _bound: IsTransparentWrapper<Outer, Inner>,
 ) -> &[Inner] {
     __check_same_alignment! {Outer, Inner}
 
+    // safety: `_bound` guarantees that `Outer` has the same layout as `Inner`
     unsafe {
         __priv_transmute_slice! {Outer, Inner, reff}
     }
